#ifndef RMSCHEDULER_H
#define RMSCHEDULER_H

#include "task.h"

static inline void rm_schedule( Task* t, int n, int h )
{
    int selected = -1;

    task_sort_rate( t, 3 );
    task_print( t, 3 );

    // the period is the deadline - therefore a task can either be
    // STOPPED, READY to be scheduled or RUNNING

    // We schedule over one hyper-period
    for( int i = 0; i < h; i++ ) {
        // check if tasks are ready ( period reached )
        // set the state to READY and the remtime to the exectime
        // additionally check if the deadline is violated
        for( int k = 0; k < n; k++ ) {
            if( i % ( t + k )->period == 0 ) {
                if( ( t + k )->state != TASK_STOPPED )
                    printf( "Error: deadline not met! Task is either ready or running!" );
                ( t + k )->state = TASK_READY;
                ( t + k )->remtime = ( t + k )->exectime;
            }
        }
        // select task with highest priority
        // schedule task with the highest priority and being ready/running
        // if a different task was running set its state to READY
        for( int k = 0; k < n; k++ ) {
            if( ( t + k )->state != TASK_STOPPED ) {
                if( k != selected && selected >= 0 ) {
                    // selected task is not the currently running task
                    ( t + selected )->state = TASK_READY;
                }
                selected = k;
                ( t + selected )->state = TASK_RUNNING;
                break;
            }
        }

        // Schedule the task and print the currently running task
        printf("%d: %d\n", i, selected + 1 );
        //printf("------- %i ----------\n", i );
        //task_print( t, 3 );

        // if a task was scheduled change its remaining time and
        // if everyting is done set the state to STOPPED
        if( selected >= 0 ) {
            ( t + selected )->remtime--;
            // task has finished
            if( ! ( t + selected )->remtime ) {
                ( t + selected )->state = TASK_STOPPED;
                selected = -1;
            }
        }
    }
}

#endif
